Attribute VB_Name = "Dir_Read"
'=================================================================
' Module : DIR-READ.BAS - Directory Reader Utility
' Author : Steve Colvin
' Date   : October 20, 1996
' Purpose: To provide a means of retrieving an alphabetical list
'          of subdirectories at all subordinate levels under a
'          specified starting path.  It exposes an 'initialize'
'          and a 'getnext' function for flattening out the
'          subdirectory hierarchy allowing it to be read
'          sequentially.  Organizing the utility's functions in
'          this manner requires the processing loop (DO...LOOP)
'          to be contained within the using VB app, giving the
'          VB app complete control after each directory read
'          iteration.
'=================================================================

Type SubDirComponents
   AnchorPath As String
   DirIndex As Long
   DirCount As Long
End Type

Dim iiCrntLvl As Integer
Dim theArray() As SubDirComponents
Const cMAX_LEVELS = 100

Dim theDirListBox As DirListBox

Function GetNextSubDir(PathOut As String) As Integer
'=================================================================
' Purpose: Get the next subdirectory in the directory tree
' Input  : None
' Output : Path
' Returns: Found status (False means EOF)
'=================================================================

Dim lbContinue As Integer
Dim liWorkLvl As Integer

Do
   'Determine if there are more subdirectories to process (check all 'active' levels)
   lbContinue = False
   For liWorkLvl = 0 To iiCrntLvl
      If theArray(liWorkLvl).DirIndex < theArray(liWorkLvl).DirCount - 1 Then
         lbContinue = True
         Exit For
      End If
   Next liWorkLvl
   If Not lbContinue Then
      GetNextSubDir = False
      Exit Function
   End If
   
   If theArray(iiCrntLvl).DirIndex = theArray(iiCrntLvl).DirCount - 1 Then
      'End of subdirectories for this level - back up 1 level and restart loop
      iiCrntLvl = iiCrntLvl - 1
   Else
      'More subdirectories for this level, i.e. current directory contains more subdirectories
      
      'Re-establish path at anchor
      theDirListBox.Path = theArray(iiCrntLvl).AnchorPath
      
      'Increment directory index at current level and set path to the directory at the new index
      theArray(iiCrntLvl).DirIndex = theArray(iiCrntLvl).DirIndex + 1
      theDirListBox.Path = theDirListBox.List(theArray(iiCrntLvl).DirIndex)
      
      'drill down to next level
      iiCrntLvl = iiCrntLvl + 1
      SetArrayForCrntLvl
      Exit Do
   End If
Loop
PathOut = theDirListBox.Path
GetNextSubDir = True
End Function

Function InitSubDirSearch(PathIn As String, DirectoryListBox As DirListBox) As Integer
'=================================================================
' Purpose: Initialize work environment for sequential access
'          of a directory tree for a given starting path.
' Input  : Starting Path
'          Directory ListBox (preferably invisible)
' Output : None
' Returns: Found status (False means Starting Path not found)
'=================================================================

On Error GoTo InitSubDirSearchError

'Establish Directory ListBox
Set theDirListBox = Nothing
Set theDirListBox = DirectoryListBox
theDirListBox.Path = PathIn

'Initialize array
ReDim theArray(cMAX_LEVELS)
iiCrntLvl = 0
SetArrayForCrntLvl
InitSubDirSearch = True
Exit Function

InitSubDirSearchError:
InitSubDirSearch = False
Exit Function
End Function

Private Sub SetArrayForCrntLvl()
'=================================================================
' Purpose: Set contents of array for the current level (iiCrntLvl)
' Input  : None
' Output : None
' Returns: None
'=================================================================

theArray(iiCrntLvl).AnchorPath = theDirListBox.Path
theArray(iiCrntLvl).DirIndex = -1
theArray(iiCrntLvl).DirCount = theDirListBox.ListCount
End Sub

